 
var productViewer = function(element) {
    this.element = element;
    this.handleContainer = this.element.find('.cd-product-viewer-handle');
    this.handleFill = this.handleContainer.children('.fill');
    this.handle = this.handleContainer.children('.handle');
    this.imageWrapper = this.element.find('.product-viewer');
    this.slideShow = this.imageWrapper.children('.product-sprite');
    this.frames = this.element.data('frame');
    this.friction = this.element.data('friction');
    this.visibleFrame = 0;
    this.lastFrame = 0;
    this.loaded = false;
    this.animating = false;
    this.xPosition = 0;
    this.notableframes=[];
  
    this.loadFrames();
}
productViewer.prototype.loadFrames = function() {
    var self = this,
        imageUrl = this.slideShow.data('image'),
        newImg = $('<img/>');
    this.loading('0.5');
    newImg.attr('src', imageUrl).load(function() {
        $(this).remove();
        self.loaded = true;
    }).each(function() {
        image = this;
        if (image.complete) {
            $(image).trigger('load');
        }
    });
}
productViewer.prototype.loading = function(percentage) {
    var self = this;
    transformElement(this.handleFill, 'scaleX(' + percentage + ')');
    setTimeout(function() {
        if (self.loaded) {
            self.element.addClass('loaded');
            transformElement(self.handleFill, 'scaleX(1)');
            self.dragImage();
            if (self.handle) self.dragHandle();
        } else {
            var newPercentage = parseFloat(percentage) + .1;
            if (newPercentage < 1) {
                self.loading(newPercentage);
            }
        }
    }, 500);
}
productViewer.prototype.dragHandle = function() {
    var self = this;
    self.handle.on('mousedown vmousedown', function(e) {
        self.handle.addClass('cd-draggable');
        var dragWidth = self.handle.outerWidth(),
            containerOffset = self.handleContainer.offset().left,
            containerWidth = self.handleContainer.outerWidth(),
            minLeft = containerOffset - dragWidth / 2,
            maxLeft = containerOffset + containerWidth - dragWidth / 2;
        self.xPosition = self.handle.offset().left + dragWidth - e.pageX;
        self.element.on('mousemove vmousemove', function(e) {
            if (!self.animating) {
                self.animating = true;
                (!window.requestAnimationFrame) ? setTimeout(function() {
                    self.animateDraggedHandle(e, dragWidth, containerOffset, containerWidth, minLeft, maxLeft);
                }, 100): requestAnimationFrame(function() {
                    self.animateDraggedHandle(e, dragWidth, containerOffset, containerWidth, minLeft, maxLeft);
                });
            }
        }).one('mouseup vmouseup', function(e) {
            self.handle.removeClass('cd-draggable');
            self.element.off('mousemove vmousemove');
        });
        e.preventDefault();
    }).on('mouseup vmouseup', function(e) {
        self.handle.removeClass('cd-draggable');
    });
}
productViewer.prototype.animateDraggedHandle = function(e, dragWidth, containerOffset, containerWidth, minLeft, maxLeft) {
    var self = this;
    var leftValue = e.pageX + self.xPosition - dragWidth;
    if (leftValue < minLeft) {
        leftValue = minLeft;
    } else if (leftValue > maxLeft) {
        leftValue = maxLeft;
    }
    var widthValue = Math.ceil((leftValue + dragWidth / 2 - containerOffset) * 1000 / containerWidth) / 10;
    self.visibleFrame = Math.ceil((widthValue * (self.frames - 1)) / 100);
    self.updateFrame();
    $('.cd-draggable', self.handleContainer).css('left', widthValue + '%').one('mouseup vmouseup', function() {
        $(this).removeClass('cd-draggable');
    });
    self.animating = false;
}
productViewer.prototype.dragImage = function() {
    var self = this;
    self.slideShow.on('mousedown vmousedown', function(e) {
        self.slideShow.addClass('cd-draggable');
        var containerOffset = self.imageWrapper.offset().left,
            containerWidth = self.imageWrapper.outerWidth(),
            minFrame = 0,
            maxFrame = self.frames - 1;
        self.xPosition = e.pageX;
        self.element.on('mousemove vmousemove', function(e) {
            if (!self.animating) {
                self.animating = true;
                (!window.requestAnimationFrame) ? setTimeout(function() {
                    self.animateDraggedImage(e, containerOffset, containerWidth);
                }, 100): requestAnimationFrame(function() {
                    self.animateDraggedImage(e, containerOffset, containerWidth);
                });
            }
        }).one('mouseup vmouseup', function(e) {
            self.slideShow.removeClass('cd-draggable');
            self.element.off('mousemove vmousemove');
            self.updateHandle();
        });
        e.preventDefault();
    }).on('mouseup vmouseup', function(e) {
        self.slideShow.removeClass('cd-draggable');
    });
}
productViewer.prototype.animateDraggedImage = function(e, containerOffset, containerWidth) {
    var self = this;
    var leftValue = self.xPosition - e.pageX;
    var widthValue = Math.ceil((leftValue) * 100 / (containerWidth * self.friction));
    var frame = (widthValue * (self.frames - 1)) / 100;
    if (frame > 0) {
        frame = Math.floor(frame);
    } else {
        frame = Math.ceil(frame);
    }
    var newFrame = self.visibleFrame + frame;
    if (newFrame < 0) {
        newFrame = self.frames - 1;
    } else if (newFrame > self.frames - 1) {
        newFrame = 0;
    }
    if (newFrame != self.visibleFrame) {
        self.visibleFrame = newFrame;
        self.updateFrame();
        self.xPosition = e.pageX;
       
    }
    self.animating = false;
}
productViewer.prototype.updateHandle = function() {
    if (this.handle) {
        var widthValue = 100 * this.visibleFrame / this.frames;
        this.handle.animate({
            'left': widthValue + '%'
        }, 200);
    }
}
productViewer.prototype.updateFrame = function() {
    var transformValue = -(100 * this.visibleFrame / this.frames);
    transformElement(this.slideShow, 'translateX(' + transformValue + '%)');
    if(this.visibleFrame != this.lastFrame){
        this.displayInfo(this.visibleFrame);
        this.lastFrame = this.visibleFrame;
        console.log(this.visibleFrame);
    }
    
}


productViewer.prototype.displayInfo = function(frame) {
    
    $('.info3d').each(function(){
            $(this).hide();
    });
    if(jQuery.inArray(frame, this.notableframes)!=-1){
        
        $('.info'+frame).each(function(){
          
            $(this).show();
        });
    }
}
productViewer.prototype.addNotableFrame = function(frameNumberIni, frameNumberEnd, x, y, side, text) {
   


    for(i = frameNumberIni; i<=frameNumberEnd; i++){

        var pos = jQuery.inArray(i, this.notableframes);
        var count = 0;

        this.notableframes.push(i);

        if(pos!=-1){
          
            for(var j = 0; j<this.notableframes.length; j++){
                if(this.notableframes[j]==i) count++;
            }
        }


        if(side == 'l'){
            orientacao = 'left';
        }
        else if(side == 'r'){
            orientacao = 'right';
        }
        else if(side == 'u'){
            orientacao = 'up';
        }
        else if(side == 'd'){
            orientacao = 'down';
        }
        else{
            console.error('Nenhuma orientação válida inserida (Foi inserida a orientação "'+side+'"). Orientações válidas: "l" - left, "r"-right, "u"-up e "d"-down.');
            continue;
        }

        _html = '<div class="info3d info'+i+' info'+i+'-'+count+' info3d-'+orientacao+'"><span>'+text+'</span></div>';

        $(_html).appendTo('.product-viewer');

        $('.info'+i+'-'+count).css({'left':x+'%', 'top':y+'%'});

        
    }
    this.displayInfo(this.visibleFrame);
 
}   




function transformElement(element, value) {
    element.css({
        '-moz-transform': value,
        '-webkit-transform': value,
        '-ms-transform': value,
        '-o-transform': value,
        'transform': value,
    });
}

